import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:nuntio_hera/hera/hera.dart';
import 'package:nuntio_hera/hera/models/find_options.dart';
import 'package:nuntio_hera/hera/models/user_options.dart';
import 'package:nuntio_hera/hera_ui/AuthPage/VerifyCodePage/verify_code_page.dart';
import 'package:nuntio_hera/hera_ui/components/nuntio_button.dart';
import 'package:nuntio_hera/hera_ui/components/nuntio_text_field.dart';
import 'package:nuntio_hera/hera_ui/models.dart';
import '../../../../hera.pb.dart';

class PasswordInfoPage extends StatefulWidget {
  PasswordInfoPage({
    Key? key,
    required this.nuntioStyle,
    required this.nuntioText,
    required this.nuntioColor,
    required this.nuntioTextStyle,
    required this.logo,
    required this.config,
    required this.background,
    required this.nuntioFooter,
    required this.email,
    required this.username,
    required this.phoneNumber,
    required this.onRegister,
    required this.onLogin,
    this.brightness,
  }) : super(key: key);

  // style and text config
  final HeraStyle nuntioStyle;
  final HeraText nuntioText;
  final HeraColor nuntioColor;
  final HeraTextStyle nuntioTextStyle;
  final HeraFooter nuntioFooter;

  // general
  final Widget logo;
  final Config config;
  final Brightness? brightness;

  // style
  final BoxDecoration background;

  // user
  final String username;
  final String email;
  final String phoneNumber;

  // functions
  final Function(BuildContext context) onRegister;
  final Function(BuildContext context) onLogin;

  @override
  State<PasswordInfoPage> createState() => _PasswordInfoPageState();
}

class _PasswordInfoPageState extends State<PasswordInfoPage> {
  final _passwordController = TextEditingController();
  final _repeatPasswordController = TextEditingController();

  bool isLoading = false;
  bool containsEightCharacters = false;
  bool containsNumber = false;
  bool containsSpecial = false;
  bool passwordMatch = false;

  @override
  void initState() {
    containsEightCharacters = !widget.config.validatePassword;
    containsNumber = !widget.config.validatePassword;
    containsSpecial = !widget.config.validatePassword;
    super.initState();
  }

  onPasswordChange() {
    if (_repeatPasswordController.text != "" &&
        _passwordController.text != "" &&
        _repeatPasswordController.text == _passwordController.text) {
      setState(() {
        passwordMatch = true;
      });
    } else {
      setState(() {
        passwordMatch = false;
      });
    }
    if (!widget.config.validatePassword) {
      return;
    }
    if (_passwordController.text != "") {
      if (_passwordController.text.contains(RegExp(r'[0-9]'))) {
        if (!containsNumber) {
          setState(() {
            containsNumber = true;
          });
        }
      } else {
        setState(() {
          containsNumber = false;
        });
      }
      if (_passwordController.text
          .contains(RegExp(r'[!@#$%^&*(),.?":{}|<>]'))) {
        if (!containsSpecial) {
          setState(() {
            containsSpecial = true;
          });
        }
      } else {
        setState(() {
          containsSpecial = false;
        });
      }
      if (_passwordController.text.length > 8) {
        if (!containsEightCharacters) {
          setState(() {
            containsEightCharacters = true;
          });
        }
      } else {
        setState(() {
          containsEightCharacters = false;
        });
      }
    } else {
      setState(() {
        containsEightCharacters = false;
        containsSpecial = false;
        containsNumber = false;
        passwordMatch = false;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    var size = MediaQuery.of(context).size;
    return CupertinoPageScaffold(
      child: SingleChildScrollView(
        child: Container(
          decoration: widget.background,
          constraints: BoxConstraints(minHeight: size.height),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              SizedBox(),
              Center(
                child: Container(
                  padding: EdgeInsets.only(top: 30),
                  constraints: BoxConstraints(maxWidth: 450),
                  margin: const EdgeInsets.only(
                      left: 20, right: 20, bottom: 20, top: 0),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.center,
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: <Widget>[
                      widget.logo,
                      if (widget.nuntioText.registerTitle != "")
                        Padding(
                          padding: const EdgeInsets.only(bottom: 10),
                          child: Text(
                            widget.nuntioText.registerTitle,
                            style: widget.nuntioTextStyle.titleStyle,
                            textAlign: TextAlign.center,
                          ),
                        ),
                      Text(
                        widget.nuntioText.registerPasswordSubtitle,
                        style: widget.nuntioTextStyle.subtitleStyle,
                        textAlign: TextAlign.center,
                      ),
                      NuntioTextField(
                        nuntioTextStyle: widget.nuntioTextStyle,
                        nuntioStyle: widget.nuntioStyle,
                        nuntioColor: widget.nuntioColor,
                        controller: _passwordController,
                        hint: widget.nuntioText.passwordLabelText,
                        textInputType: TextInputType.text,
                        label: widget.nuntioText.passwordLabelText,
                        textInputAction: TextInputAction.next,
                        onChanged: (_) => {onPasswordChange()},
                        obscureText: true,
                        prefix: FontAwesomeIcons.lock,
                        brightness: widget.brightness,
                      ),
                      NuntioTextField(
                        nuntioTextStyle: widget.nuntioTextStyle,
                        nuntioStyle: widget.nuntioStyle,
                        nuntioColor: widget.nuntioColor,
                        controller: _repeatPasswordController,
                        hint: widget.nuntioText.repeatPasswordLabelText,
                        textInputType: TextInputType.text,
                        label: widget.nuntioText.repeatPasswordLabelText,
                        obscureText: true,
                        textInputAction: TextInputAction.done,
                        prefix: FontAwesomeIcons.repeat,
                        onChanged: (_) => {onPasswordChange()},
                        brightness: widget.brightness,
                      ),
                      if (widget.config.validatePassword)
                        Column(
                          crossAxisAlignment: CrossAxisAlignment.center,
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: [
                            const SizedBox(height: 10),
                            Row(
                              crossAxisAlignment: CrossAxisAlignment.center,
                              children: <Widget>[
                                Icon(
                                  containsEightCharacters
                                      ? CupertinoIcons.check_mark_circled
                                      : CupertinoIcons.xmark_circle,
                                  color: containsEightCharacters
                                      ? widget.nuntioColor.successColor
                                      : widget.nuntioColor.errorColor,
                                  size: 15,
                                ),
                                SizedBox(
                                  width: 5,
                                ),
                                Text(
                                  widget.nuntioText
                                      .passwordContainsEightCharsText,
                                  style: widget.nuntioTextStyle.bodyTextStyle,
                                )
                              ],
                            ),
                            SizedBox(
                              height: 10,
                            ),
                            Row(
                              crossAxisAlignment: CrossAxisAlignment.center,
                              children: <Widget>[
                                Icon(
                                  containsNumber
                                      ? CupertinoIcons.check_mark_circled
                                      : CupertinoIcons.xmark_circle,
                                  color: containsNumber
                                      ? widget.nuntioColor.successColor
                                      : widget.nuntioColor.errorColor,
                                  size: 15,
                                ),
                                SizedBox(
                                  width: 5,
                                ),
                                Text(
                                  widget.nuntioText.passwordContainsNumberText,
                                  style: widget.nuntioTextStyle.bodyTextStyle,
                                )
                              ],
                            ),
                            SizedBox(
                              height: 10,
                            ),
                            Row(
                              crossAxisAlignment: CrossAxisAlignment.center,
                              children: <Widget>[
                                Icon(
                                  containsSpecial
                                      ? CupertinoIcons.check_mark_circled
                                      : CupertinoIcons.xmark_circle,
                                  color: containsSpecial
                                      ? widget.nuntioColor.successColor
                                      : widget.nuntioColor.errorColor,
                                  size: 15,
                                ),
                                SizedBox(
                                  width: 5,
                                ),
                                Text(
                                  widget.nuntioText.passwordContainsSpecialText,
                                  style: widget.nuntioTextStyle.bodyTextStyle,
                                )
                              ],
                            ),
                            SizedBox(
                              height: 10,
                            ),
                            Row(
                              crossAxisAlignment: CrossAxisAlignment.center,
                              children: <Widget>[
                                Icon(
                                  passwordMatch
                                      ? CupertinoIcons.check_mark_circled
                                      : CupertinoIcons.xmark_circle,
                                  color: passwordMatch
                                      ? widget.nuntioColor.successColor
                                      : widget.nuntioColor.errorColor,
                                  size: 15,
                                ),
                                SizedBox(
                                  width: 5,
                                ),
                                Text(
                                  widget.nuntioText.passwordsMustMatchText,
                                  style: widget.nuntioTextStyle.bodyTextStyle,
                                )
                              ],
                            ),
                          ],
                        ),
                      const SizedBox(height: 20),
                      SizedBox(
                        width: widget.nuntioStyle.buttonWidth,
                        height: widget.nuntioStyle.buttonHeight,
                        child: NuntioButton(
                          color: widget.nuntioColor.secondaryColor,
                          filled: true,
                          disabled: isLoading ||
                              !containsSpecial ||
                              !containsNumber ||
                              !containsEightCharacters ||
                              !passwordMatch,
                          disabledTipnote:
                              widget.nuntioText.invalidPasswordText,
                          onPressed: onRegister,
                          child: Row(
                            crossAxisAlignment: CrossAxisAlignment.center,
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              Text(
                                widget.nuntioText.registerButtonText,
                                style: widget
                                    .nuntioTextStyle.registerButtonTextStyle,
                                textAlign: TextAlign.center,
                              ),
                              SizedBox(width: 8),
                              FaIcon(
                                FontAwesomeIcons.arrowRight,
                                size: 15,
                                color: widget.nuntioTextStyle
                                    .registerButtonTextStyle.color,
                              ),
                            ],
                          ),
                        ),
                      ),
                      NuntioButton(
                        child: Text(
                          "Go back",
                        ),
                        onPressed: () => {Navigator.of(context).pop()},
                        color: widget.nuntioColor.primaryColor,
                      ),
                    ],
                  ),
                ),
              ),
              SizedBox(
                height: widget.nuntioFooter.height,
                child: widget.nuntioFooter.footer,
              )
            ],
          ),
        ),
      ),
    );
  }

  afterLoginFailure() {
    _passwordController.text = "";
    _repeatPasswordController.text = "";
    setState(() {
      isLoading = false;
      containsEightCharacters = false;
      containsNumber = false;
      containsSpecial = false;
      passwordMatch = false;
    });
  }

  afterLoginSuccess() {
    _passwordController.text = "";
    _repeatPasswordController.text = "";
    setState(() {
      isLoading = false;
      containsEightCharacters = false;
      containsNumber = false;
      containsSpecial = false;
      passwordMatch = false;
    });
    widget.onRegister(context);
  }

  onRegister() {
    setState(() {
      isLoading = true;
    });
    Hera.createUser(
        password: _passwordController.text,
        userOptions: UserOptions(
          email: widget.email,
          phone: widget.phoneNumber,
          username: widget.username,
        )).catchError((err) {
      _passwordController.text = "";
      _repeatPasswordController.text = "";
      setState(() {
        isLoading = false;
        containsEightCharacters = false;
        containsNumber = false;
        containsSpecial = false;
        passwordMatch = false;
      });
      showCupertinoDialog(
          context: context,
          builder: (context) {
            return CupertinoAlertDialog(
              title: Text(
                widget.nuntioText.errorTitle,
              ),
              content: Text(
                widget.nuntioText.errorSubtitle,
              ),
              actions: <Widget>[
                CupertinoButton(
                    onPressed: () {
                      Navigator.of(context).pop();
                    },
                    child: const Text(
                      'okay',
                      style: TextStyle(color: black),
                    ))
              ],
            );
          });
      return err;
    }).then((user) {
      // todo: handle error
      Hera.login(
              password: _passwordController.text,
              findOptions: FindOptions(email: widget.email, phone: widget.phoneNumber, username: widget.username))
          .then((loginSession) {
        if (loginSession.loginStatus == LoginStatus.EMAIL_IS_NOT_VERIFIED) {
          Navigator.of(context)
              .push(
            CupertinoPageRoute(
              builder: (context) => VerifyCodePage(
                loginType: LoginType.EMAIL_PASSWORD,
                identifier: user.email,
                emailExpiresAt:
                    loginSession.emailExpiresAt.toDateTime().toUtc(),
                config: widget.config,
                onLogin: widget.onLogin,
                nuntioStyle: widget.nuntioStyle,
                nuntioColor: widget.nuntioColor,
                background: widget.background,
                nuntioText: widget.nuntioText,
                nuntioFooter: widget.nuntioFooter,
                nuntioTextStyle: widget.nuntioTextStyle,
                password: _passwordController.text,
                logo: widget.logo,
              ),
            ),
          )
              .then((_) {
            setState(() {
              isLoading = false;
            });
            _passwordController.text = "";
            _repeatPasswordController.text = "";
          });
        } else {
          afterLoginSuccess();
        }
      });
    });
  }
}
