import 'package:flutter/cupertino.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:nuntio_hera/hera_ui/AuthPage/RegisterPage/PasswordInfoPage/password_info_page.dart';
import 'package:nuntio_hera/hera_ui/components/nuntio_button.dart';
import 'package:nuntio_hera/hera_ui/components/nuntio_text_field.dart';
import 'package:nuntio_hera/hera_ui/functions.dart';
import 'package:nuntio_hera/hera_ui/models.dart';
import '../../../../hera.pb.dart';
import '../../../../hera.pbenum.dart';

class RegisterPage extends StatefulWidget {
  RegisterPage({
    Key? key,
    required this.nuntioStyle,
    required this.nuntioText,
    required this.nuntioColor,
    required this.nuntioTextStyle,
    required this.logo,
    required this.config,
    required this.background,
    required this.nuntioFooter,
    required this.supportedAuthMechanisms,
    required this.onRegister,
    required this.onLogin,
    this.brightness,
  }) : super(key: key);

  // style and text config
  final List<LoginType> supportedAuthMechanisms;
  final HeraStyle nuntioStyle;
  final HeraText nuntioText;
  final HeraColor nuntioColor;
  final HeraTextStyle nuntioTextStyle;
  final HeraFooter nuntioFooter;

  // general
  final Widget logo;
  final Config config;
  final Brightness? brightness;

  // style
  final BoxDecoration background;

  // functions
  final Function(BuildContext context) onRegister;
  final Function(BuildContext context) onLogin;

  @override
  State<RegisterPage> createState() => _RegisterPage();
}

class _RegisterPage extends State<RegisterPage> {
  final _usernameController = TextEditingController();
  final _emailController = TextEditingController();
  final _phoneController = TextEditingController();
  String disabledTipnote = "";

  void getDisabledTipnote() {
    if (widget.supportedAuthMechanisms.contains(LoginType.USERNAME_PASSWORD) &&
        _usernameController.text == "") {
      setState(() {
        disabledTipnote = widget.nuntioText.invalidUsernameText;
      });
    } else if (widget.supportedAuthMechanisms
            .contains(LoginType.EMAIL_PASSWORD) &&
        validateEmail(_emailController.text) == false) {
      setState(() {
        disabledTipnote = widget.nuntioText.invalidEmailText;
      });
    } else if (widget.supportedAuthMechanisms
            .contains(LoginType.PHONE_PASSWORD) &&
        validateMobile(_phoneController.text) == false) {
      setState(() {
        disabledTipnote = widget.nuntioText.invalidPhoneText;
      });
    } else {
      setState(() {
        disabledTipnote = "";
      });
    }
    return;
  }

  onSubmit() {
    Navigator.of(context).push(CupertinoPageRoute(
        builder: (context) => PasswordInfoPage(
              nuntioStyle: widget.nuntioStyle,
              nuntioText: widget.nuntioText,
              nuntioColor: widget.nuntioColor,
              nuntioTextStyle: widget.nuntioTextStyle,
              logo: widget.logo,
              config: widget.config,
              background: widget.background,
              nuntioFooter: widget.nuntioFooter,
              phoneNumber: _phoneController.text,
              email: _emailController.text,
              username: _usernameController.text,
              onLogin: widget.onLogin,
              onRegister: widget.onLogin,
            )));
  }

  @override
  void initState() {
    super.initState();
    getDisabledTipnote();
  }

  @override
  Widget build(BuildContext context) {
    var size = MediaQuery.of(context).size;
    return CupertinoPageScaffold(
      child: SingleChildScrollView(
        child: Container(
          decoration: widget.background,
          constraints: BoxConstraints(minHeight: size.height),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              SizedBox(),
              Center(
                child: Container(
                  padding: EdgeInsets.only(top: 30),
                  constraints: BoxConstraints(maxWidth: 450),
                  margin: const EdgeInsets.only(
                      left: 20, right: 20, bottom: 20, top: 0),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.center,
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: <Widget>[
                      widget.logo,
                      if (widget.nuntioText.registerTitle != "")
                        Padding(
                          padding: const EdgeInsets.only(bottom: 10),
                          child: Text(
                            widget.nuntioText.registerTitle,
                            style: widget.nuntioTextStyle.titleStyle,
                            textAlign: TextAlign.center,
                          ),
                        ),
                      Text(
                        widget.nuntioText.registerContactSubtitle,
                        style: widget.nuntioTextStyle.subtitleStyle,
                        textAlign: TextAlign.center,
                      ),
                      SizedBox(height: 30),
                      if (widget.config.supportedLoginMechanisms
                          .contains(LoginType.USERNAME_PASSWORD))
                        NuntioTextField(
                          nuntioTextStyle: widget.nuntioTextStyle,
                          nuntioStyle: widget.nuntioStyle,
                          onChanged: (_) {
                            getDisabledTipnote();
                          },
                          nuntioColor: widget.nuntioColor,
                          controller: _usernameController,
                          hint: "JohnDoe1234",
                          textInputType: TextInputType.text,
                          label: widget.nuntioText.usernameLabelText,
                          textInputAction: widget
                                      .config.supportedLoginMechanisms
                                      .contains(LoginType.EMAIL_PASSWORD) &&
                                  widget.config.supportedLoginMechanisms
                                      .contains(LoginType.PHONE_PASSWORD)
                              ? TextInputAction.next
                              : TextInputAction.done,
                          onSubmitted: (_) {
                            if (widget.config.supportedLoginMechanisms
                                    .contains(LoginType.EMAIL_PASSWORD) &&
                                widget.config.supportedLoginMechanisms
                                    .contains(LoginType.PHONE_PASSWORD)) {
                              return;
                            }
                            onSubmit();
                          },
                          prefix: FontAwesomeIcons.hatCowboy,
                          brightness: widget.brightness,
                        ),
                      if (widget.config.supportedLoginMechanisms
                          .contains(LoginType.EMAIL_PASSWORD))
                        NuntioTextField(
                          nuntioTextStyle: widget.nuntioTextStyle,
                          nuntioStyle: widget.nuntioStyle,
                          nuntioColor: widget.nuntioColor,
                          controller: _emailController,
                          onChanged: (_) {
                            getDisabledTipnote();
                          },
                          hint: "mail@website.com",
                          textInputType: TextInputType.emailAddress,
                          label: widget.nuntioText.emailLabelText,
                          textInputAction: widget
                                  .config.supportedLoginMechanisms
                                  .contains(LoginType.PHONE_PASSWORD)
                              ? TextInputAction.next
                              : TextInputAction.done,
                          prefix: FontAwesomeIcons.paperPlane,
                          onSubmitted: (_) {
                            if (widget.config.supportedLoginMechanisms
                                    .contains(LoginType.PHONE_PASSWORD)) {
                              return;
                            }
                            onSubmit();
                          },
                          brightness: widget.brightness,
                        ),
                      if (widget.config.supportedLoginMechanisms
                          .contains(LoginType.PHONE_PASSWORD))
                        NuntioTextField(
                          nuntioTextStyle: widget.nuntioTextStyle,
                          nuntioStyle: widget.nuntioStyle,
                          nuntioColor: widget.nuntioColor,
                          controller: _phoneController,
                          onChanged: (_) {
                            getDisabledTipnote();
                          },
                          onSubmitted: (_) {
                            onSubmit();
                          },
                          hint: "999-999-999",
                          textInputType: TextInputType.phone,
                          label: widget.nuntioText.phoneLabelText,
                          textInputAction: TextInputAction.done,
                          prefix: FontAwesomeIcons.phone,
                          brightness: widget.brightness,
                        ),
                      const SizedBox(
                        height: 30,
                      ),
                      SizedBox(
                        width: widget.nuntioStyle.buttonWidth,
                        height: widget.nuntioStyle.buttonHeight,
                        child: NuntioButton(
                          color: widget.nuntioColor.secondaryColor,
                          filled: true,
                          disabledTipnote: disabledTipnote,
                          disabled: disabledTipnote != "",
                          onPressed: () => {onSubmit()},
                          child: Text(
                            widget.nuntioText.nextButtonText,
                            style:
                                widget.nuntioTextStyle.registerButtonTextStyle,
                            textAlign: TextAlign.center,
                          ),
                        ),
                      ),
                      NuntioButton(
                        child: Text(
                          widget.nuntioText.alreadyHaveAccountButtonText,
                        ),
                        onPressed: () => {
                          Navigator.of(context).pop(),
                        },
                        color: widget.nuntioColor.primaryColor,
                      ),
                    ],
                  ),
                ),
              ),
              SizedBox(
                height: widget.nuntioFooter.height,
                child: widget.nuntioFooter.footer,
              )
            ],
          ),
        ),
      ),
    );
  }
}
